@@ -22,19 +22,20 @@
  *
  */
 
+#include "global.h"
+#include "io.h"
+#include "process.h"
+#include "sequence.h"
 #include <assert.h>
 #include <errno.h>
 #include <getopt.h>
+#include <gsl/gsl_rng.h>
 #include <limits.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
-#include <gsl/gsl_rng.h>
-#include "global.h"
-#include "process.h"
-#include "io.h"
-#include "sequence.h"
+#include <unistd.h>
 
 #ifdef _OPENMP
 #include <omp.h>
@@ -44,11 +45,12 @@
 int FLAGS = 0;
 int THREADS = 1;
 long unsigned int BOOTSTRAP = 0;
-double RANDOM_ANCHOR_PROP = 0.05;
+double RANDOM_ANCHOR_PROP = 0.025;
 gsl_rng *RNG = NULL;
 int MODEL = M_JC;
+int EXIT_CODE = EXIT_SUCCESS;
 
-void usage(void);
+void usage(int);
 void version(void);
 
 /**
@@ -60,39 +62,52 @@ void version(void);
  * and errors.
  */
 int main(int argc, char *argv[]) {
-	int c;
-	int version_flag = 0;
-
-	struct option long_options[] = {{"version", no_argument, &version_flag, 1},
-									{"help", no_argument, NULL, 'h'},
-									{"verbose", no_argument, NULL, 'v'},
-									{"join", no_argument, NULL, 'j'},
-									{"low-memory", no_argument, NULL, 'l'},
-									{"threads", required_argument, NULL, 't'},
-									{"bootstrap", required_argument, NULL, 'b'},
-									{"model", required_argument, NULL, 'm'},
-									{0, 0, 0, 0}};
+	struct option long_options[] = {
+		{"version", no_argument, NULL, 0},
+		{"truncate-names", no_argument, NULL, 0},
+		{"file-of-filenames", required_argument, NULL, 0},
+		{"help", no_argument, NULL, 'h'},
+		{"verbose", no_argument, NULL, 'v'},
+		{"join", no_argument, NULL, 'j'},
+		{"low-memory", no_argument, NULL, 'l'},
+		{"threads", required_argument, NULL, 't'},
+		{"bootstrap", required_argument, NULL, 'b'},
+		{"model", required_argument, NULL, 'm'},
+		{0, 0, 0, 0}};
 
 #ifdef _OPENMP
 	// Use all available processors by default.
 	THREADS = omp_get_num_procs();
 #endif
+	struct string_vector file_names;
+	string_vector_init(&file_names);
 
 	// parse arguments
 	while (1) {
 
 		int option_index = 0;
-
-		c = getopt_long(argc, argv, "jvht:p:m:b:l", long_options,
-						&option_index);
+		int c = getopt_long(argc, argv, "jvht:p:m:b:l", long_options,
+							&option_index);
 
 		if (c == -1) {
 			break;
 		}
 
 		switch (c) {
-			case 0: break;
-			case 'h': usage(); break;
+			case 0: {
+				const char *option_str = long_options[option_index].name;
+				if (strcasecmp(option_str, "version") == 0) {
+					version();
+				}
+				if (strcasecmp(option_str, "truncate-names") == 0) {
+					FLAGS |= F_TRUNCATE_NAMES;
+				}
+				if (strcasecmp(option_str, "file-of-filenames") == 0) {
+					read_into_string_vector(optarg, &file_names);
+				}
+				break;
+			}
+			case 'h': usage(EXIT_SUCCESS); break;
 			case 'v':
 				FLAGS |= FLAGS & F_VERBOSE ? F_EXTRA_VERBOSE : F_VERBOSE;
 				break;
@@ -182,39 +197,39 @@ int main(int argc, char *argv[]) {
 				break;
 			}
 			case '?': /* intentional fall-through */
-			default: usage(); break;
+			default: usage(EXIT_FAILURE); break;
 		}
 	}
 
-	if (version_flag) {
-		version();
-	}
-
 	argc -= optind;
 	argv += optind;
 
+	// copy command line arguments into vector
+	// std::copy, anyone?
+	for (size_t i = 0; i < (unsigned int)argc; i++) {
+		string_vector_push_back(&file_names, argv[i]);
+	}
+
 	// at least one file name must be given
-	if (FLAGS & F_JOIN && argc == 0) {
+	if (FLAGS & F_JOIN && string_vector_size(&file_names) == 0) {
 		errx(1, "In join mode at least one filename needs to be supplied.");
 	}
 
+	size_t minfiles = FLAGS & F_JOIN ? 2 : 1;
+	if (string_vector_size(&file_names) < minfiles) {
+		// not enough files passed via arguments; read from stdin.
+		string_vector_push_back(&file_names, "-");
+		// explain to the user, why nothing is happening.
+		if (isatty(STDIN_FILENO)) {
+			warnx("Not enough file names given; expecting input via stdin.");
+		}
+	}
+
+	// parse fasta files
 	dsa_t dsa;
 	dsa_init(&dsa);
-
-	const char *file_name;
-
-	// parse all files
-	int minfiles = FLAGS & F_JOIN ? 2 : 1;
-	for (;; minfiles--) {
-		if (!*argv) {
-			if (minfiles <= 0) break;
-
-			// if no files are supplied, read from stdin
-			file_name = "-";
-		} else {
-			file_name = *argv++;
-		}
-
+	for (size_t i = 0; i < string_vector_size(&file_names); i++) {
+		char *file_name = string_vector_at(&file_names, i);
 		if (FLAGS & F_JOIN) {
 			read_fasta_join(file_name, &dsa);
 		} else {
@@ -222,6 +237,8 @@ int main(int argc, char *argv[]) {
 		}
 	}
 
+	string_vector_free(&file_names);
+
 	size_t n = dsa_size(&dsa);
 
 	if (n < 2) {
@@ -242,6 +259,7 @@ int main(int argc, char *argv[]) {
 	}
 
 	// seed the random number generator with the current time
+	// TODO: enable seeding for reproducibility
 	gsl_rng_set(RNG, time(NULL));
 
 	// Warn about non ACGT residues.
@@ -253,7 +271,7 @@ int main(int argc, char *argv[]) {
 	// validate sequence correctness
 	const seq_t *seq = dsa_data(&dsa);
 	for (size_t i = 0; i < n; ++i, seq++) {
-		if (strlen(seq->name) > 10) {
+		if ((FLAGS & F_TRUNCATE_NAMES) && strlen(seq->name) > 10) {
 			warnx("The sequence name '%s' is longer than ten characters. It "
 				  "will be truncated in the output to '%.10s'.",
 				  seq->name, seq->name);
@@ -280,53 +298,58 @@ int main(int argc, char *argv[]) {
 			  "alignment instead.");
 	}
 
+	// side channel
+	EXIT_CODE = EXIT_SUCCESS;
+
 	// compute distance matrix
 	calculate_distances(dsa_data(&dsa), n);
 
 	dsa_free(&dsa);
 	gsl_rng_free(RNG);
-	return 0;
+	return EXIT_CODE;
 }
 
 /**
- * Prints the usage to stdout and then exits successfully.
+ * @brief Prints the usage and then exits.
  */
-void usage(void) {
+void usage(int status) {
 	const char str[] = {
 		"Usage: andi [-jlv] [-b INT] [-p FLOAT] [-m MODEL] [-t INT] FILES...\n"
 		"\tFILES... can be any sequence of FASTA files. If no files are "
 		"supplied, stdin is used instead.\n"
 		"Options:\n"
-		"  -b, --bootstrap <INT> \n"
-		"                    Print additional bootstrap matrices\n"
-		"  -j, --join        Treat all sequences from one file as a single "
+		"  -b, --bootstrap <INT>  Print additional bootstrap matrices\n"
+		"      --file-of-filenames <FILE>  Read additional filenames from "
+		"FILE; one per line\n"
+		"  -j, --join           Treat all sequences from one file as a single "
 		"genome\n"
-		"  -l, --low-memory  Use less memory at the cost of speed\n"
-		"  -m, --model <Raw|JC|Kimura>\n"
-		"                    Pick an evolutionary model; default: JC\n"
-		"  -p <FLOAT>        Significance of an anchor pair; default: 0.05\n"
-		"  -v, --verbose     Prints additional information\n"
+		"  -l, --low-memory     Use less memory at the cost of speed\n"
+		"  -m, --model <Raw|JC|Kimura>  Pick an evolutionary model; default: "
+		"JC\n"
+		"  -p <FLOAT>           Significance of an anchor; default: 0.025\n"
 #ifdef _OPENMP
-		"  -t, --threads <INT> \n"
-		"                    The number of threads to be used; by default, all "
+		"  -t, --threads <INT>  Set the number of threads; by default, all "
 		"available processors are used\n"
 #endif
-		"  -h, --help        Display this help and exit\n"
-		"      --version     Output version information and acknowledgments\n"};
+		"      --truncate-names Truncate names to ten characters\n"
+		"  -v, --verbose        Prints additional information\n"
+		"  -h, --help           Display this help and exit\n"
+		"      --version        Output version information and "
+		"acknowledgments\n"};
 
-	printf("%s", str);
-	exit(EXIT_SUCCESS);
+	fprintf(status == EXIT_SUCCESS ? stdout : stderr, "%s", str);
+	exit(status);
 }
 
 /**
- * This function just prints the version string and then aborts
+ * @brief This function just prints the version string and then aborts
  * the program. It conforms to the [GNU Coding
  * Standard](http://www.gnu.org/prep/standards/html_node/_002d_002dversion.html#g_t_002d_002dversion).
  */
 void version(void) {
 	const char str[] = {
 		"andi " VERSION "\n"
-		"Copyright (C) 2014 - 2016 Fabian Klötzl\n"
+		"Copyright (C) 2014 - 2017 Fabian Klötzl\n"
 		"License GPLv3+: GNU GPL version 3 or later "
 		"<http://gnu.org/licenses/gpl.html>\n"
 		"This is free software: you are free to change and redistribute it.\n"
