@@ -13,11 +13,11 @@
  * ESA. If we simply store the interval for "AA" in the cache, once and use it
  * for each query we are significantly faster (up to 7 times).
  */
-#include <stdlib.h>
-#include <string.h>
-#include <assert.h>
 #include "esa.h"
 #include "global.h"
+#include <assert.h>
+#include <stdlib.h>
+#include <string.h>
 
 static void esa_init_cache_dfs(esa_s *, char *str, size_t pos, lcp_inter_t in);
 static void esa_init_cache_fill(esa_s *, char *str, size_t pos, lcp_inter_t in);
@@ -311,7 +311,7 @@ int esa_init_CLD(esa_s *C) {
 	saidx_t *CLD = C->CLD = malloc((C->len + 1) * sizeof(*CLD));
 	CHECK_MALLOC(CLD);
 
-	saidx_t *LCP = C->LCP;
+	const saidx_t *LCP = C->LCP;
 
 	typedef struct pair_s { saidx_t idx, lcp; } pair_t;
 
@@ -365,7 +365,7 @@ int esa_init_CLD(esa_s *C) {
  */
 int esa_init_LCP(esa_s *C) {
 	const char *S = C->S;
-	saidx_t *SA = C->SA;
+	const saidx_t *SA = C->SA;
 	saidx_t len = C->len;
 
 	// Trivial safety checks
