@@ -4,18 +4,143 @@
  */
 #define _GNU_SOURCE
 #include <fcntl.h>
-#include <math.h>
 #include <limits.h>
+#include <math.h>
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
 
-#include <pfasta.h>
+#include <compat-stdlib.h>
 #include <compat-string.h>
+#include <pfasta.h>
 
 #include "global.h"
 #include "io.h"
 
+/**
+ * @brief Access an element.
+ * @param sv - The base vector.
+ * @param index - The element index to access.
+ * @returns the string at position `index`.
+ */
+char *string_vector_at(struct string_vector *sv, size_t index) {
+	return sv->data[index];
+}
+
+/**
+ * @brief Access the underlying buffer.
+ * @param sv - The base vector.
+ * @returns the underlying buffer.
+ */
+char **string_vector_data(struct string_vector *sv) {
+	return sv->data;
+}
+
+/**
+ * @brief Free all data.
+ * @param sv - The base vector.
+ */
+void string_vector_free(struct string_vector *sv) {
+	size_t i = 0;
+	for (; i < sv->size; i++) {
+		free(sv->data[i]);
+	}
+	free(sv->data);
+}
+
+/**
+ * @brief Initialise the vector.
+ * @param sv - The base vector.
+ */
+void string_vector_init(struct string_vector *sv) {
+	sv->data = malloc(sizeof(*sv->data) * 4);
+	CHECK_MALLOC(sv->data);
+
+	sv->capacity = 4;
+	sv->size = 0;
+}
+
+/**
+ * @brief Adds a copy to the end of the vector.
+ * @param sv - The base vector.
+ * @param str - The new string to add.
+ */
+void string_vector_push_back(struct string_vector *sv, const char *str) {
+	string_vector_emplace_back(sv, strdup(str));
+}
+
+/**
+ * @brief Add a file name to the end of the vector, directly.
+ * @param sv - The base vector.
+ * @param str - The string to emplace.
+ */
+void string_vector_emplace_back(struct string_vector *sv, char *str) {
+	if (sv->size < sv->capacity) {
+		sv->data[sv->size++] = str;
+	} else {
+		char **ptr = reallocarray(sv->data, sv->capacity / 2, 3 * sizeof(*ptr));
+		CHECK_MALLOC(ptr);
+		sv->data = ptr;
+		sv->capacity = (sv->capacity / 2) * 3;
+		sv->data[sv->size++] = str;
+	}
+}
+
+/**
+ * @brief Return the number of elements.
+ * @param sv - The base vector.
+ * @returns the size of the vector.
+ */
+size_t string_vector_size(const struct string_vector *sv) {
+	return sv->size;
+}
+
+/**
+ * @brief Read a *fof* and add it's contents into a vector.
+ * @param file_name - The file of file names aka. fof.
+ * @param sv - The vector to add file names to.
+ */
+void read_into_string_vector(const char *file_name, struct string_vector *sv) {
+	FILE *file = strcmp(file_name, "-") ? fopen(file_name, "r") : stdin;
+	if (!file) {
+		err(errno, "%s", file_name);
+	}
+
+	while (1) {
+		char *str = NULL;
+		size_t buffer_size = 0;
+		ssize_t check = getline(&str, &buffer_size, file);
+
+		// EOF is set only *after* getline tried to read past it.
+		if (check == -1 && feof(file) != 0) {
+			free(str);
+			break; // EOF
+		}
+
+		if (check == -1) {
+			err(errno, "%s", file_name);
+		}
+
+		char *nl = strchr(str, '\n');
+		if (nl) {
+			*nl = '\0'; // remove newline character
+		}
+
+		// ignore empty lines
+		if (strlen(str) == 0) {
+			free(str);
+			continue;
+		}
+
+		string_vector_emplace_back(sv, str);
+	}
+
+	int check = fclose(file);
+	if (check != 0) {
+		err(errno, "%s", file_name);
+	}
+}
+
 /**
  * @brief Joins all sequences from a file into a single long sequence.
  *
@@ -154,6 +279,7 @@ void print_distances(const struct model *D, const seq_t *sequences, size_t n,
 			double coverage = model_coverage(&datum);
 
 			if (isnan(dist) && warnings) {
+				EXIT_CODE = EXIT_FAILURE;
 				const char str[] = {
 					"For the two sequences '%s' and '%s' the distance "
 					"computation failed and is reported as nan. "
@@ -173,9 +299,10 @@ void print_distances(const struct model *D, const seq_t *sequences, size_t n,
 
 	printf("%zu\n", n);
 	for (i = 0; i < n; i++) {
-		// Print exactly ten characters of the name. Pad with spaces if
-		// necessary.
-		printf("%-10.10s", sequences[i].name);
+		// Print ten characters of the name. Pad with spaces, if
+		// necessary. Truncate to exactly ten characters if requested by user.
+		printf(FLAGS & F_TRUNCATE_NAMES ? "%-10.10s" : "%-10s",
+			   sequences[i].name);
 
 		for (j = 0; j < n; j++) {
 			// use scientific notation for small numbers
