@@ -4,21 +4,18 @@
  * @file
  * @brief This file contains various distance methods.
  */
+#include "process.h"
+#include "esa.h"
+#include "global.h"
+#include "io.h"
+#include "model.h"
+#include "sequence.h"
 #include <assert.h>
 #include <math.h>
 #include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include "esa.h"
-#include "global.h"
-#include "io.h"
-#include "model.h"
-#include "process.h"
-#include "sequence.h"
-
-#include <gsl/gsl_rng.h>
-#include <gsl/gsl_randist.h>
 
 #ifdef _OPENMP
 #include <omp.h>
@@ -39,13 +36,13 @@ int calculate_bootstrap(const struct model *M, const seq_t *sequences,
  * @param l - The length of the subject.
  * @returns The minimum length of an anchor.
  */
-size_t minAnchorLength(double p, double g, size_t l) {
+size_t min_anchor_length(double p, double g, size_t l) {
 	size_t x = 1;
 
 	double prop = 0.0;
-	while (prop < 1 - p) {
+	// Find smallest x with P(X > x) < p
+	for (; prop < 1 - p; x++) {
 		prop = shuprop(x, g / 2, l);
-		x++;
 	}
 
 	return x;
@@ -54,10 +51,10 @@ size_t minAnchorLength(double p, double g, size_t l) {
 /**
  * @brief Calculates the binomial coefficient of n and k.
  *
- * We could (and probalby should) use gsl_sf_lnchoose(xx,kk) for this.
+ * We could (and probably should) use gsl_sf_lnchoose(xx,kk) for this.
  *
  * @param n - The n part of the binomial coefficient.
- * @param k - analog.
+ * @param k - analogue.
  * @returns (n choose k)
  */
 size_t binomial_coefficient(size_t n, size_t k) {
@@ -92,7 +89,7 @@ size_t binomial_coefficient(size_t n, size_t k) {
  * set. See Haubold et al. (2009).
  *
  * @param x - The maximum length of a shustring.
- * @param g - The the half of the relative amount of GC in the DNA.
+ * @param p - The half of the relative amount of GC in the DNA.
  * @param l - The length of the subject.
  * @returns The probability of a certain shustring length.
  */
@@ -127,11 +124,12 @@ double shuprop(size_t x, double p, size_t l) {
  * substrings that exist in both sequences. Then it manually checks for
  * mutations between those anchors.
  *
- * @return An estimate for the number of mutations within homologous regions.
  * @param C - The enhanced suffix array of the subject.
  * @param query - The actual query string.
  * @param query_length - The length of the query string. Needed for speed
  * reasons.
+ * @param gc - The relative GC content of the subject.
+ * @returns A matrix with estimates of base substitutions.
  */
 model dist_anchor(const esa_s *C, const char *query, size_t query_length,
 				  double gc) {
@@ -152,51 +150,24 @@ model dist_anchor(const esa_s *C, const char *query, size_t query_length,
 
 	size_t num_right_anchors = 0;
 
-#ifdef DEBUG
-	size_t num_matches = 0;
-	size_t num_anchors = 0;
-	size_t num_anchors_in_rc = 0;
-	size_t num_right_anchors_in_rc = 0;
-	size_t length_anchors = 0;
-	double off_num = 0.0;
-	double off_dem = 0.0;
-#endif
-
-	size_t threshold =
-		minAnchorLength(1 - sqrt(1 - RANDOM_ANCHOR_PROP), gc, C->len);
+	size_t threshold = min_anchor_length(RANDOM_ANCHOR_PROP, gc, C->len);
 
 	// Iterate over the complete query.
 	while (this_pos_Q < query_length) {
 		inter =
 			get_match_cached(C, query + this_pos_Q, query_length - this_pos_Q);
 
-#ifdef DEBUG
-		num_matches++;
-#endif
-
 		this_length = inter.l <= 0 ? 0 : inter.l;
 
 		if (inter.i == inter.j && this_length >= threshold) {
 			// We have reached a new anchor.
 			this_pos_S = C->SA[inter.i];
 
-#ifdef DEBUG
-			num_anchors++;
-			length_anchors += this_length;
-			if (this_pos_S < (size_t)(C->len / 2)) {
-				num_anchors_in_rc++;
-			}
-#endif
-
 			// Check if this can be a right anchor to the last one.
 			if (this_pos_S > last_pos_S &&
 				this_pos_Q - last_pos_Q == this_pos_S - last_pos_S) {
 				num_right_anchors++;
-#ifdef DEBUG
-				if (this_pos_S < (size_t)(C->len / 2)) {
-					num_right_anchors_in_rc++;
-				}
-#endif
+
 				// classify nucleotides in the qanchor
 				model_count_equal(&ret, query + last_pos_Q, last_length);
 
@@ -206,19 +177,11 @@ model dist_anchor(const esa_s *C, const char *query, size_t query_length,
 							this_pos_Q - last_pos_Q - last_length);
 				last_was_right_anchor = 1;
 			} else {
-#ifdef DEBUG
-				double off = fabs((double)(this_pos_Q - last_pos_Q) -
-								  (double)(this_pos_S - last_pos_S));
-				if (off < 100) {
-					off_num += off;
-					off_dem++;
-				}
-#endif
 				if (last_was_right_anchor) {
 					// If the last was a right anchor, but with the current one,
 					// we cannot extend, then add its length.
 					model_count_equal(&ret, query + last_pos_Q, last_length);
-				} else if ((last_length / 2) >= threshold) {
+				} else if (last_length >= threshold * 2) {
 					// The last anchor wasn't neither a left or right anchor.
 					// But, it was as long as an anchor pair. So still count it.
 					model_count_equal(&ret, query + last_pos_Q, last_length);
@@ -237,27 +200,6 @@ model dist_anchor(const esa_s *C, const char *query, size_t query_length,
 		this_pos_Q += this_length + 1;
 	}
 
-#ifdef DEBUG
-	if (FLAGS & F_EXTRA_VERBOSE) {
-		const char str[] = {"- threshold: %ld\n"
-							"- matches: %lu\n"
-							"- anchors: %lu (rc: %lu)\n"
-							"- right anchors: %lu (rc: %lu)\n"
-							"- avg length: %lf\n"
-							"- off: %f (skipped: %.0f)\n"
-							"\n"};
-
-#pragma omp critical
-		{
-			fprintf(stderr, str, threshold, num_matches, num_anchors,
-					num_anchors_in_rc, num_right_anchors,
-					num_right_anchors_in_rc,
-					(double)length_anchors / num_anchors, off_num / off_dem,
-					off_dem);
-		}
-	}
-#endif
-
 	// Very special case: The sequences are identical
 	if (last_length >= query_length) {
 		model_count(&ret, C->S + last_pos_S, query, query_length);
@@ -265,31 +207,22 @@ model dist_anchor(const esa_s *C, const char *query, size_t query_length,
 	}
 
 	// We might miss a few nucleotides if the last anchor was also a right
-	// anchor.
+	// anchor. The logic is the same as a few lines above.
 	if (last_was_right_anchor) {
 		model_count(&ret, C->S + last_pos_S, query + last_pos_Q, last_length);
+	} else if (last_length >= threshold * 2) {
+		model_count_equal(&ret, query + last_pos_Q, last_length);
 	}
 
 	return ret;
 }
 
-/**
- * @brief Computes the distance matrix.
- *
- * The distMatrix() populates the D matrix with computed distances.
- * @param sequences An array of pointers to the sequences.
- * @param n The number of sequences.
+/*
+ * Include distMatrix and distMatrixLM.
  */
 #define FAST
 #include "dist_hack.h"
 
-/**
- * @brief Computes the distance matrix.
- *
- * The distMatrixLM() populates the D matrix with computed distances.
- * @param sequences An array of pointers to the sequences.
- * @param n The number of sequences.
- */
 #undef FAST
 #include "dist_hack.h"
 
@@ -343,7 +276,7 @@ void calculate_distances(seq_t *sequences, size_t n) {
 /** Yet another hack. */
 #define B(X, Y) (B[(X)*n + (Y)])
 
-/** @brief Computes a bootstrap from _pairwise_ aligments.
+/** @brief Computes a bootstrap from _pairwise_ alignments.
  *
  * Doing bootstrapping for alignments with only two sequences is easy. It boils
  * down to a simple multi-nomial process over the substitution matrix.
